"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isOracleStaleOrUnconfident = exports.isPythOracle = exports.isSwitchboardOracle = exports.parseSwitchboardOracle = exports.parseSwitchboardOracleV2 = exports.switchboardDecimalToBig = exports.parseSwitchboardOracleV1 = exports.StubOracle = exports.OracleProvider = void 0;
const client_1 = require("@pythnetwork/client");
const web3_js_1 = require("@solana/web3.js");
const sbv2_lite_1 = __importDefault(require("@switchboard-xyz/sbv2-lite"));
const big_js_1 = __importDefault(require("big.js"));
const I80F48_1 = require("../numbers/I80F48");
const SBV1_DEVNET_PID = new web3_js_1.PublicKey('7azgmy1pFXHikv36q1zZASvFq5vFa39TT9NweVugKKTU');
const SBV1_MAINNET_PID = new web3_js_1.PublicKey('DtmE9D2CSB4L5D6A15mraeEjrGMm6auWVzgaD8hK2tZM');
let sbv2DevnetProgram;
let sbv2MainnetProgram;
var OracleProvider;
(function (OracleProvider) {
    OracleProvider[OracleProvider["Pyth"] = 0] = "Pyth";
    OracleProvider[OracleProvider["Switchboard"] = 1] = "Switchboard";
    OracleProvider[OracleProvider["Stub"] = 2] = "Stub";
})(OracleProvider = exports.OracleProvider || (exports.OracleProvider = {}));
class StubOracle {
    publicKey;
    group;
    mint;
    lastUpdateTs;
    lastUpdateSlot;
    price;
    deviation;
    static from(publicKey, obj) {
        return new StubOracle(publicKey, obj.group, obj.mint, obj.price, obj.lastUpdateTs, obj.lastUpdateSlot, obj.deviation);
    }
    constructor(publicKey, group, mint, price, lastUpdateTs, lastUpdateSlot, deviation) {
        this.publicKey = publicKey;
        this.group = group;
        this.mint = mint;
        this.lastUpdateTs = lastUpdateTs;
        this.lastUpdateSlot = lastUpdateSlot;
        this.price = I80F48_1.I80F48.from(price);
        this.deviation = I80F48_1.I80F48.from(deviation);
    }
}
exports.StubOracle = StubOracle;
// https://gist.github.com/microwavedcola1/b741a11e6ee273a859f3ef00b35ac1f0
function parseSwitchboardOracleV1(accountInfo) {
    const price = accountInfo.data.readDoubleLE(1 + 32 + 4 + 4);
    const lastUpdatedSlot = parseInt(accountInfo.data.readBigUInt64LE(1 + 32 + 4 + 4 + 8).toString());
    const minResponse = accountInfo.data.readDoubleLE(1 + 32 + 4 + 4 + 8 + 8 + 8);
    const maxResponse = accountInfo.data.readDoubleLE(1 + 32 + 4 + 4 + 8 + 8 + 8 + 8);
    return { price, lastUpdatedSlot, uiDeviation: maxResponse - minResponse };
}
exports.parseSwitchboardOracleV1 = parseSwitchboardOracleV1;
function switchboardDecimalToBig(sbDecimal) {
    const mantissa = new big_js_1.default(sbDecimal.mantissa.toString());
    const scale = sbDecimal.scale;
    const oldDp = big_js_1.default.DP;
    big_js_1.default.DP = 20;
    const result = mantissa.div(new big_js_1.default(10).pow(scale));
    big_js_1.default.DP = oldDp;
    return result;
}
exports.switchboardDecimalToBig = switchboardDecimalToBig;
function parseSwitchboardOracleV2(program, accountInfo, oracle) {
    try {
        //
        const price = program.decodeLatestAggregatorValue(accountInfo).toNumber();
        const lastUpdatedSlot = program
            .decodeAggregator(accountInfo)
            .latestConfirmedRound.roundOpenSlot.toNumber();
        const stdDeviation = switchboardDecimalToBig(program.decodeAggregator(accountInfo).latestConfirmedRound.stdDeviation);
        return { price, lastUpdatedSlot, uiDeviation: stdDeviation.toNumber() };
        //if oracle is badly configured or didn't publish price at least once
        //decodeLatestAggregatorValue can throw (0 switchboard rounds).
    }
    catch (e) {
        console.log(`Unable to parse Switchboard Oracle V2: ${oracle}`, e);
        return { price: 0, lastUpdatedSlot: 0, uiDeviation: 0 };
    }
}
exports.parseSwitchboardOracleV2 = parseSwitchboardOracleV2;
/**
 *
 * @param accountInfo
 * @returns ui price
 */
async function parseSwitchboardOracle(oracle, accountInfo, connection) {
    if (accountInfo.owner.equals(sbv2_lite_1.default.devnetPid)) {
        if (!sbv2DevnetProgram) {
            sbv2DevnetProgram = await sbv2_lite_1.default.loadDevnet(connection);
        }
        return parseSwitchboardOracleV2(sbv2DevnetProgram, accountInfo, oracle);
    }
    if (accountInfo.owner.equals(sbv2_lite_1.default.mainnetPid)) {
        if (!sbv2MainnetProgram) {
            sbv2MainnetProgram = await sbv2_lite_1.default.loadMainnet(connection);
        }
        return parseSwitchboardOracleV2(sbv2MainnetProgram, accountInfo, oracle);
    }
    if (accountInfo.owner.equals(SBV1_DEVNET_PID) ||
        accountInfo.owner.equals(SBV1_MAINNET_PID)) {
        return parseSwitchboardOracleV1(accountInfo);
    }
    throw new Error(`Should not be reached!`);
}
exports.parseSwitchboardOracle = parseSwitchboardOracle;
function isSwitchboardOracle(accountInfo) {
    if (accountInfo.owner.equals(SBV1_DEVNET_PID) ||
        accountInfo.owner.equals(SBV1_MAINNET_PID) ||
        accountInfo.owner.equals(sbv2_lite_1.default.devnetPid) ||
        accountInfo.owner.equals(sbv2_lite_1.default.mainnetPid)) {
        return true;
    }
    return false;
}
exports.isSwitchboardOracle = isSwitchboardOracle;
function isPythOracle(accountInfo) {
    return accountInfo.data.readUInt32LE(0) === client_1.Magic;
}
exports.isPythOracle = isPythOracle;
function isOracleStaleOrUnconfident(nowSlot, maxStalenessSlots, oracleLastUpdatedSlot, deviation, confFilter, price) {
    if (maxStalenessSlots >= 0 &&
        oracleLastUpdatedSlot &&
        nowSlot > oracleLastUpdatedSlot + maxStalenessSlots) {
        return true;
    }
    if (deviation && deviation.gt(confFilter.mul(price))) {
        return true;
    }
    return false;
}
exports.isOracleStaleOrUnconfident = isOracleStaleOrUnconfident;
